Strict
Import mojo


#if CONFIG = "debug"
Const DEBUG:Int = 1
	
#else
Const DEBUG:Int = 0
	
#endif




'Size of theoretical gamespace window, not actual screen size.
Const _cGameWidth:Float = 800
Const _cGameHeight:Float = 600


Global _gTimePassed:Float
Global _gLastTime:Float
Global _gDelta:Float


Global _gaKeyTappedRegister:Int[127]
Global _gKeyLockedRegister:Int


'Tempory loadspace for _gImageSpritesheet image, automatically removed once cut-up into stored animations.
Global _gImageSpritesheet:Image


'Animated mushroom.
Global _gAnimMushroom:C_Animation


'Animated centipede segments.
Global _gAnimCentipedeHead:C_Animation
Global _gAnimCentipedeBody:C_Animation
Global _gAnimCentipedeTail:C_Animation


'Animated projectile.
Global _gAnimProjectile:C_Animation


'AnimatedPlayer.
Global _gAnimPlayer:C_Animation


Global _gaCentipedeList:C_Centipede[]
Global _gaMushroomList:C_Mushroom[]
Global _gaProjectileList:C_Projectile[]
Global _gPlayer:C_Player


Global _gTitleOn:Int
Global _gGameOn:Int


Global gDebugMessage:String[]




Function Main:Int()
	New C_Game
	Return 0
	
End








Class C_Game Extends App
	Method OnCreate:Int()
		SetUpdateRate(60)
		
		_fInitiate()
		_fLoadGraphics()
		_fCreateAnimations()
		_fUnloadGraphics()
		
		_fCreatePlayer(21 * 18, 21 * 27, 0, 1)
		_fPopulateMushrooms()
		
		_fCreateCentipede(_cGameWidth, 21, 1, 10, 5.00, 1, 2)
'		_fCreateCentipede(400, 21 * 27, 3, 10, 5.00, 0)
'		_fCreateCentipede(21 * 40, 0, 1, 10, 5.00, 2)
		
		Return 0
		
	End
	
	
	Method OnUpdate:Int()
		_gTimePassed = (Millisecs() - _gLastTime)
		_gLastTime = Millisecs()
		_gDelta = (_gTimePassed / 1000.0)
		
		if _gTimePassed > 300
			_gTimePassed = 0
			_gLastTime = Millisecs()
			_gDelta = 0
			
		EndIf
		
		_fUpdateGame()
		
		if _fKeyTapped(KEY_SPACE) = True
			_fPopulateMushrooms()
				
		EndIf
		
		Return 0
		
	End
	
	
	Method OnRender:Int()
		Cls(0, 0, 0)
		_fDrawGame()
		
		if DEBUG = 1
			_fDrawDebugMessages(0, 0)
			
		EndIf
		
		Return 0
		
	End
	
End




'#Region _Classes
Class C_Timer
	Field delay:Float
	Field lastTime:Float
	
	
	Method New(delay:Float)
		SetDelay(delay)
		Self.lastTime = Millisecs()
		
	End
	
	
	Method GetTrigger:Bool()
		Local trigger:Bool
		
		
		if Millisecs() >= (Self.lastTime + Self.delay)
			Self.lastTime = Millisecs()
			trigger = True
		Else
			trigger = False
		EndIf
		
		Return trigger
		
	End
	
	
	Method SetDelay:Void(delay:Float)
		Self.delay = delay
		Self.lastTime = Millisecs()
		
	End
	
	
	Method GetDelay:Float()
		Return Self.delay
		
	End
	
	
	Method GetTimePassed:Float()
		Local timePassed:Float = (Millisecs() - Self.lastTime)
		
		
		if timePassed > delay
			timePassed = delay
			Self.lastTime = Millisecs()
			
		EndIf
		
		Return timePassed
		
	End
	
End




Class C_Rectangle
	Field absX:Float, absY:Float, width:Float, height:Float
	
	
	Method New(x:Float = 0, y:Float = 0, width:Float = 0, height:Float = 0)
		Self.SetAbsX(x)
		Self.SetAbsY(y)
		Self.SetWidth(width)
		Self.SetHeight(height)
		
	End
	
	
	Method SetAbsX:Void(x:Float)
		Self.absX = x
		
	End
	
	
	Method SetAbsY:Void(y:Float)
		Self.absY = y
		
	End
	
	
	Method SetWidth:Void(width:Float)
		Self.width = width
		
	End
	
	
	Method SetHeight:Void(height:Float)
		Self.height = height
		
	End
	
	
	Method GetAbsX:Float()
		Return Self.absX
		
	End
	
	
	Method GetAbsY:Float()
		Return Self.absY
		
	End
	
	
	Method GetWidth:Float()
		Return Self.width
		
	End
	
	
	Method GetHeight:Float()
		Return Self.height
		
	End
	
End




Class C_Animation
	Field imageList:Image[]
	Field currentFrame:Int
	Field frameTimer:C_Timer
	Field auto:Int
	
	
	Method New(sourceImage:Image, frameList:Int[], frameTime:Float)
		Local i:Int
		Local sX:Int, sY:Int, sWidth:Float, sHeight:Float
		
		Self.frameTimer = New C_Timer(frameTime)
		Self.currentFrame = 0
		
		For i = 0 to ((frameList.Length() / 4) - 1)
			sX = frameList[i * 4]
			sY = frameList[(i * 4) + 1]
			sWidth = frameList[(i * 4) + 2]
			sHeight = frameList[(i * 4) + 3]
			
			Self.imageList = Self.imageList.Resize(Self.imageList.Length() + 1)
			Self.imageList[i] = _fFrameGrab(sourceImage, sX, sY, sWidth, sHeight, 1)
			
		Next
		
	End
	
	
	Method SetAuto:Void(auto:Int)
		Self.auto = auto
		
	End
	
	
	Method Draw:Void(x:Float, y:Float, rotation:Float)
		_fDrawImageToScale(Self.imageList[Self.currentFrame], x, y, rotation)
		
	End
	
	
	Method IncreaseCurrentFrame:Void()
		if Self.auto = 0
			Self.currentFrame = (Self.currentFrame + 1)
			
			if Self.currentFrame > (Self.imageList.Length() - 1)
				Self.currentFrame = (Self.imageList.Length() - 1)
				
			EndIf
			
		EndIf
		
	End
	
	
	Method DecreaseCurrentFrame:Void()
		if Self.auto = 0
			Self.currentFrame = (Self.currentFrame - 1)
			
			if Self.currentFrame < 0
				Self.currentFrame = 0
				
			EndIf
			
		EndIf
		
	End
	
	
	Method Update:Void()
		if Self.auto = 1
			if Self.frameTimer.GetTrigger() = True
				Self.currentFrame = (Self.currentFrame + 1)
				
				if Self.currentFrame > (Self.imageList.Length() - 1)
					Self.currentFrame = 0
					
				EndIf
				
			EndIf
			
		End
	End
	
End




Class C_Player
	Field absX:Float, absY:Float
	Field speed:Float
	Field direction:Int
	Field animation:C_Animation
	
	
	Method New(x:Float = 0, y:Float = 0, direction:Int = 0, speed:Float = 0)
		Self.SetAbsX(x)
		Self.SetAbsY(y)
		Self.SetSpeed(speed)
		Self.SetAnim(_gAnimPlayer)
		
	End
	
	
	Method SetAbsX:Void(x:Float)
		Self.absX = x
		
	End
	
	
	Method SetAbsY:Void(y:Float)
		Self.absY = y
		
	End
	
	
	Method SetSpeed:Void(speed:Float)
		Self.speed = speed
		
	End
	
	
	Method SetDirection:Void(direction:Float)
		Self.direction = direction
		
	End
	
	
	Method SetAnim:Void(animation:C_Animation)
		Self.animation = animation
		
	End
	
	
	Method GetAbsX:Float()
		Return Self.absX
		
	End
	
	
	Method GetAbsY:Float()
		Return Self.absY
		
	End
	
	
	Method GetSpeed:Float()
		Return Self.speed
		
	End
	
	
	Method GetDirection:Int()
		Return Self.direction
		
	End
	
	
	Method GetWidth:Float()
		Return Self.animation.imageList[0].Width()
		
	End
	
	
	Method GetHeight:Float()
		Return Self.animation.imageList[0].Height()
		
	End
	
	
	Method Draw:Void()
		Self.animation.Draw(Self.GetAbsX() + (Self.GetWidth() / 2), Self.GetAbsY() + (Self.GetHeight() / 2), Self.GetDirection() * 90)
		
	End
	
	
	Method Update:Void()
		
	End
	
End





Class C_Projectile
	Field absX:Float, absY:Float
	Field speed:Float
	Field direction:Int
	Field animation:C_Animation
	Field active:Int
	
	
	Method New(x:Float, y:Float, speed:Float, direction:Int)
		Self.SetAbsX(x)
		Self.SetAbsY(y)
		Self.SetSpeed(speed)
		Self.SetDirection(direction)
		Self.SetAnim(_gAnimProjectile)
		Self.SetActive(1)
		
	End
	
	
	Method SetAbsX:Void(x:Float)
		Self.absX = x
		
	End
	
	
	Method SetAbsY:Void(y:Float)
		Self.absY = y
		
	End
	
	
	Method SetSpeed:Void(speed:Float)
		Self.speed = speed
		
	End
	
	
	Method SetDirection:Void(direction:Int)
		Self.direction = direction
		
	End
	
	
	Method SetActive:Void(active:Int)
		Self.active = active
		
	End
	
	
	Method SetAnim:Void(animation:C_Animation)
		Self.animation = animation
		
	End
	
	
	Method GetAbsX:Float()
		Return Self.absX
		
	End
	
	
	Method GetAbsY:Float()
		Return Self.absY
		
	End
	
	
	Method GetSpeed:Float()
		Return Self.speed
		
	End
	
	
	Method GetDirection:Int()
		Return Self.direction
		
	End
	
	
	Method GetActive:Int()
		Return Self.active
		
	End
	
	
	Method GetWidth:Float()
		Return Self.animation.imageList[0].Width()
		
	End
	
	
	Method GetHeight:Float()
		Return Self.animation.imageList[0].Height()
		
	End
	
	
	Method Move:Void(dlt:Float)
		Local pixelsPerSecond:Float = (dlt * 100)
		
		
		if Self.GetDirection() = 0
			Self.SetAbsY(Self.GetAbsY() - (pixelsPerSecond * Self.GetSpeed()))
			
		EndIf
		
		if Self.GetDirection() = 1
			Self.SetAbsX(Self.GetAbsX() - (pixelsPerSecond * Self.GetSpeed()))
			
		EndIf
		
		if Self.GetDirection() = 2
			Self.SetAbsY(Self.GetAbsY() + (pixelsPerSecond * Self.GetSpeed()))
			
		EndIf
		
		if Self.GetDirection() = 3
			Self.SetAbsX(Self.GetAbsX() + (pixelsPerSecond * Self.GetSpeed()))
			
		EndIf
		
	End
	
	
	Method Draw:Void()
		Self.animation.Draw(Self.GetAbsX() + (Self.GetWidth() / 2), Self.GetAbsY() + (Self.GetHeight() / 2), Self.GetDirection() * 90)
		
	End
	
	
	Method Update:Void(dlt:Float)
		Self.Move(dlt)
		
	End
	
End




Class C_Mushroom
	Field absX:Float, absY:Float, width:Float, height:Float
	Field targetX:Float, targetY:Float
	Field animation:C_Animation
	Field health:Float
	Field alive:Int
	
	
	'	Field boundBox:C_Rectangle
	
	
	Method New(x:Float, y:Float, health:Float = 5)
		SetPos(x, y)
		Self.SetAnimation(_gAnimMushroom)
		Self.SetHealth(health)
		Self.alive = 1
		
	End
	
	
	Method SetPos:Void(x:Float, y:Float)
		Self.SetAbsX(x)
		Self.SetAbsY(y)
		
	End
	
	
	Method SetHealth:Void(health:Float)
		Self.health = health
		
	End
	
	
	Method DecreaseHealth:Void(amount:Float = 1)
		Self.SetHealth(Self.GetHealth() - amount)
		
		if Self.GetHealth() < 0
			Self.SetHealth(0)
			Self.alive = 0
			
		EndIf
		
	End
	
	
	Method IncreaseHealth:Void(amount:Float = 1.00)
		if amount > 0.00 And Self.GetHealth() < animation.imageList.Length()
			Self.SetHealth(Self.GetHealth() + amount)
			alive = 1
			
		EndIf
		
	End
	
	
	Method SetAbsX:Void(x:Float)
		Self.absX = x
		
	End
	
	
	Method SetAbsY:Void(y:Float)
		Self.absY = y
		
	End
	
	
	Method SetAnimation:Void(animation:C_Animation)
		Self.animation = animation
		
	End
	
	
	Method GetHealth:Float()
		Return Self.health
		
	End
	
	Method GetAbsX:Float()
		Return Self.absX
		
	End
	
	
	Method GetAbsY:Float()
		Return Self.absY
		
	End
	
	
	Method GetWidth:Float()
		Return animation.imageList[0].Width()
			
	End
	
	
	Method GetHeight:Float()
		Return animation.imageList[0].Height()
		
	End
	
	
	'	Method GetBoundBox:C_Rectangle()
	'		if Self.boundBox = Null
	'			Self.boundBox = New C_Rectangle()
	'
	'		EndIf
	'
	'		boundBox.SetAbsX(Self.GetAbsX())
	'		boundBox.SetAbsY(Self.GetAbsY())
	'		boundBox.SetWidth(Self.GetWidth())
	'		boundBox.SetHeight(Self.GetHeight())
	'
	'		Return boundBox
	'
	'	End
	
	
	Method Draw:Void()
		if self.GetHealth() > 0
			self.animation.currentFrame = (Self.animation.imageList.Length() - 1) - (Self.GetHealth() - 1)
			Self.animation.Draw(Self.GetAbsX() + (Self.GetWidth() / 2), Self.GetAbsY() + (Self.GetHeight() / 2), 0)
			
		EndIf
		
		if DEBUG = 1
			_fDrawSquare(Self.GetAbsX(), Self.GetAbsY(), Self.GetWidth(), Self.GetHeight())
			_fDrawTextAtScale(Self.GetHealth(), Self.GetAbsX() + Self.GetWidth(), Self.GetAbsY())
			
		EndIf
		
	End
	
End




Class C_Centipede
	Field partsList:C_CentepedePart[]
	Field tick:Float
	Field tickTrigger:Float
	Field speed:Float
	Field active:Int
	
	
	Method New(x:Float, y:Float, direction:Int, length:int, speed:Float = 1, progressionX:Int, progressionY:Int)
		Local i:Int
		Local xSpacing:Int = 0
		Local ySpacing:Int = 0
		Local spacingOffset:Float = 5
		
		
		Self.tick = 0
		Self.tickTrigger = 1.00
		Self.SetSpeed(speed)
		Self.partsList = Self.partsList.Resize(length)
		Self.SetActive(1)
		
		
		if direction = 0
			ySpacing = (_gAnimCentipedeHead.imageList[_gAnimCentipedeHead.currentFrame].Height() - spacingOffset)
			
		EndIf
		
		if direction = 1
			xSpacing = (_gAnimCentipedeHead.imageList[_gAnimCentipedeHead.currentFrame].Width() - spacingOffset)
			
		EndIf
		
		if direction = 2
			ySpacing = -(_gAnimCentipedeHead.imageList[_gAnimCentipedeHead.currentFrame].Height() - spacingOffset)
			
		EndIf
		
		if direction = 3
			xSpacing = -(_gAnimCentipedeHead.imageList[_gAnimCentipedeHead.currentFrame].Width() - spacingOffset)
			
		EndIf
		
		'Create head
		Self.partsList[0] = New C_CentepedePart(x, y, direction, "head", speed)
		Self.partsList[0].SetProgressionX(progressionX)
		Self.partsList[0].SetProgressionY(progressionY)
		
		'Create body
		For i = 1 to (length - 2)
			Self.partsList[i] = New C_CentepedePart(x + (xSpacing * i), y + (ySpacing * i), direction, "body", speed)
			Self.partsList[i].SetProgressionX(progressionX)
			Self.partsList[i].SetProgressionY(progressionY)
			
		Next
		
		'Create tail
		Self.partsList[length - 1] = New C_CentepedePart(x + (xSpacing * (length - 1)), y + (ySpacing * (length - 1)), direction, "tail", speed)
		Self.partsList[length - 1].SetProgressionX(progressionX)
		Self.partsList[length - 1].SetProgressionY(progressionY)
		
	End
	
	
	Method SetActive:Void(active:Int)
		Self.active = active
		
	End
	
	
	Method SetSpeed:Void(speed:Float)
		Self.speed = speed
		
	End
	
	
	Method GetActive:Int()
		Return Self.active
		
	End
	
	
	Method GetSpeed:Float()
		Return Self.speed
		
	End
	
	
	Method Draw:Void()
		Local i:Int
		
		
		'Draw body and tail
		For i = (Self.partsList.Length() - 1) to 1 Step -1
			if Self.partsList[i]
				Self.partsList[i].Draw()
				
			EndIf
			
		Next
		
		'Draw head
		if Self.partsList[0]
			Self.partsList[0].Draw()
			
		EndIf
		
	End
	
	
	Method CanGoUp:Int(cPart:C_CentepedePart)
		Local top:Float = 0
		Local bottom:Float = (cPart.GetHeight() * 27)
		Local left:Float = 0
		Local right:Float = (cPart.GetWidth() * 37)
		Local flag:Int = 1
		
		
		if cPart.GetAbsY() <= top or cPart.GetAbsY() >= bottom
			flag = 0
			
		EndIf
		
		if cPart.GetAbsX() <= left or cPart.GetAbsX() >= right
			flag = 0
			
		EndIf
		
		Return flag
		
	End
	
	
	Method UpdatePartsListLength:Void()
		For mushroom = EachIn _gaMushroomList
			if mushroom
				mList = mList.Resize(mList.Length() + 1)
				mList[mList.Length() - 1] = mushroom
				
			EndIf
			
		Next
		
		_gaMushroomList = []
		
		For mushroom = EachIn mList
			if mushroom
				_gaMushroomList = _gaMushroomList.Resize(_gaMushroomList.Length() + 1)
				_gaMushroomList[_gaMushroomList.Length() - 1] = mushroom
				
			EndIf
			
		Next
		
		mList = []
		
	End
	
	
	Method Update:Void(dlt:Float)
		Local pixelsPerSecond:Float = (dlt * 100)
		Local cPart:C_CentepedePart
		
		
		if Self.tick = Self.tickTrigger
			For cPart = EachIn Self.partsList
				if cPart.GetDirection() = 0
					cPart.SetBaseY(cPart.GetBaseY() - Self.GetSpeed())
					
				EndIf
				
				if cPart.GetDirection() = 1
					cPart.SetBaseX(cPart.GetBaseX() - Self.GetSpeed())
					cPart.SetProgressionX(1)
					
				EndIf
				
				if cPart.GetDirection() = 2
					cPart.SetBaseY(cPart.GetBaseY() + Self.GetSpeed())
					
				EndIf
				
				if cPart.GetDirection() = 3
					cPart.SetBaseX(cPart.GetBaseX() + Self.GetSpeed())
					cPart.SetProgressionX(3)
					
				EndIf
				
			Next
			
			Self.tick = 0
			
		EndIf
		
		
		For cPart = EachIn Self.partsList
			if cPart
				cPart.Update()
				
			EndIf
			
		Next
		
		Self.tick = Self.tick + (pixelsPerSecond)
		
		if Self.tick >= Self.tickTrigger
			Self.tick = Self.tickTrigger
			
		EndIf
		
		_fCreateDebugMessage("Centipede Segments: " + Self.partsList.Length())
		
	End
		
End




Class C_CentepedePart
	Field baseX:Float, baseY:Float
	Field previousY:Float
	Field progressionX:Int
	Field progressionY:int
	Field moveOffset:Float
	Field direction:Int
	Field previousDirection:Int
	Field speed:Float
	Field partID:String
	Field animation:C_Animation
	Field boundBox:C_Rectangle
	Field active:Int
	
	
	Method New(x:Float, y:Float, direction:Int, partID:String, speed:Float = 1)
		Self.SetDirection(direction)
		Self.SetSpeed(speed)
		Self.SetPartID(partID)
		Self.SetPos(x, y)
		Self.SetMoveOffset(0)
		Self.SetActive(1)
		Self.previousY = y
		
	End
	
	
	Method SetPartID:Void(partID:String)
		partID = partID.ToLower()
		Self.partID = partID
		
		if partID.Contains("h")
			Self.SetAnim(_gAnimCentipedeHead)
			
		EndIf
		
		if partID.Contains("b")
			Self.SetAnim(_gAnimCentipedeBody)
			
		EndIf
		
		if partID.Contains("t")
			Self.SetAnim(_gAnimCentipedeTail)
			
		EndIf
		
	End
	
	
	Method SetActive:Void(active:Int)
		Self.active = active
		
	End
	
	
	Method SetPos:Void(x:Float, y:Float)
		Self.SetBaseX(x)
		Self.SetBaseY(y)
		
	End
	
	
	Method SetSpeed:Void(speed:Float)
		Self.speed = speed
		
	End
	
	
	Method SetProgressionX:Void(direction:Int)
		Self.progressionX = direction
		
	End
	
	
	Method SetProgressionY:Void(direction:Int)
		Self.progressionY = direction
		
	End
	
	
	Method SetBaseX:Void(x:Float)
		Self.baseX = x
		
	End
	
	
	Method SetBaseY:Void(y:Float)
		Self.baseY = y
		
	End
	
	
	Method SetMoveOffset:Void(offset:Float)
		Self.moveOffset = offset
		
	End
	
	
	Method SetDirection:Void(direction:Int)
		Self.previousDirection = Self.direction
		Self.direction = direction
		
	End
	
	
	Method SetAnim:Void(animation:C_Animation)
		Self.animation = animation
		
	End
	
	
	Method GetActive:Int()
		Return Self.active
		
	End
	
	
	Method GetDirection:Int()
		Return Self.direction
		
	End
	
	
	Method GetProgressionX:Int()
		Return Self.progressionX
		
	End
	
	
	Method GetProgressionY:Int()
		Return Self.progressionY
		
	End
	
	
	Method GetBaseX:Float()
		Return Self.baseX
		
	End
	
	
	Method GetBaseY:Float()
		Return Self.baseY
		
	End
	
	
	Method GetMoveOffset:Float()
		Return Self.moveOffset
		
	End
	
	
	Method GetAbsX:Float()
		Local flag:Float
		
		
		if Self.GetDirection() = 1
			flag = (Self.GetBaseX() - (Self.GetWidth() * Self.GetMoveOffset()))
			
		ElseIf Self.GetDirection() = 3
			flag = (Self.GetBaseX() + (Self.GetWidth() * Self.GetMoveOffset()))
			
		Else
			flag = Self.GetBaseX()
			
		EndIf
		
		Return flag
		
	End
	
	
	Method GetAbsY:Float()
		Local flag:Float
		
		
		if Self.GetDirection() = 0
			flag = (Self.GetBaseY() - (Self.GetHeight() * Self.GetMoveOffset()))
			
		ElseIf Self.GetDirection() = 2
			flag = (Self.GetBaseY() + (Self.GetHeight() * Self.GetMoveOffset()))
			
		Else
			flag = Self.GetBaseY()
			
		EndIf
		
		Return flag
		
	End
	
	
	Method GetWidth:Float()
		Return animation.imageList[0].Width()
			
	End
	
	
	Method GetHeight:Float()
		Return animation.imageList[0].Height()
		
	End
	
	
	Method GetPartID:String()
		Return Self.partID
		
	End
	
	
'	Method InCheck:Int()
'		Local flag:Int = 0
'		
'		
'		if Self.GetAbsX() = Self.GetBaseX() And Self.GetAbsY() = Self.GetBaseY()
'			flag = 1
'			
'		EndIf
'		
'		Return flag
'		
'	End
	
	
	Method Draw:Void()
		Self.animation.Draw(Self.GetAbsX() + (Self.GetWidth() / 2), Self.GetAbsY() + (Self.GetHeight() / 2), Self.GetDirection() * 90)
		
		if DEBUG = 1
			_fDrawSquare(Self.GetBaseX(), Self.GetBaseY(), Self.GetWidth(), Self.GetHeight())
			
		EndIf
		
	End
	
	
	Method Update:Void()
		Self.animation.Update()
		
	End
	
End

'#End Region








'#Region __Mushroom
Function _fCreateMushroomFrames:Void()
	_gAnimMushroom = _fCreateAnimation(_gImageSpritesheet,
		[70, 0, 21, 21,
		93, 0, 21, 21,
		116, 0, 21, 21,
		139, 0, 21, 21,
		162, 0, 21, 21],
		0)
		
	_gAnimMushroom.SetAuto(0)
	
End




Function _fPopulateMushrooms:Void(density:Int = 1)
	Local x:Float, y:Float
	Local mushroomsPerRow:Int
	Local i:Int
	
	
	Seed = Millisecs()
	_fClearMushrooms()
	
	For y = 1 to 26
		mushroomsPerRow = Rnd(0, density - 1)
		
		For i = 0 to mushroomsPerRow
			x = Floor(Rnd(37))
			
			While _fMushroomExistsAtPos(x, y) = 1 or _fPlayerExistsAtPos(x, y) = 1
				x = Floor(Rnd(37))
				
			Wend
			
			_fCreateMushroom(_gAnimMushroom.imageList[0].Width() * x, _gAnimMushroom.imageList[0].Height() * y)
			
		Next
		
	Next
	
End




Function _fClearMushrooms:Void()
	_gaMushroomList = []
	
End




Function _fCreateMushroom:Void(x:Float, y:Float)
	_gaMushroomList = _gaMushroomList.Resize(_gaMushroomList.Length() + 1)
	_gaMushroomList[_gaMushroomList.Length() - 1] = New C_Mushroom(x, y)
	_gaMushroomList[_gaMushroomList.Length() - 1].animation.SetAuto(1)
	
End




Function _fMushroomExistsAtPos:Int(x:Float, y:Float)
	Local mushroom:C_Mushroom
	Local flag:Int
	
	
	For mushroom = EachIn _gaMushroomList
		if x > mushroom.GetAbsX() And x < (mushroom.GetAbsX() + mushroom.GetWidth())
			if y > mushroom.GetAbsY() And y < (mushroom.GetAbsY() + mushroom.GetHeight())
				flag = 1
				
			Else
				flag = 0
				
			EndIf
			
		EndIf
		
	Next
	
	Return flag
	
End




Function _fDrawMushrooms:Void()
	Local mushroom:C_Mushroom
	
	
	For mushroom = EachIn _gaMushroomList
		if mushroom
			mushroom.Draw()
			
		EndIf
		
	Next
	
End




Function _fUpdateMushroomListLength:Void()
	Local mushroom:C_Mushroom
	Local mList:C_Mushroom[]
	
	
	For mushroom = EachIn _gaMushroomList
		if mushroom
			mList = mList.Resize(mList.Length() + 1)
			mList[mList.Length() - 1] = mushroom
			
		EndIf
		
	Next
	
	_gaMushroomList = []
	
	For mushroom = EachIn mList
		if mushroom
			_gaMushroomList = _gaMushroomList.Resize(_gaMushroomList.Length() + 1)
			_gaMushroomList[_gaMushroomList.Length() - 1] = mushroom
			
		EndIf
		
	Next
	
	mList = []
	
End




Function _fUpdateMushrooms:Void()
	Local i:Float
	
	
	For i = 0 to (_gaMushroomList.Length() - 1)
		if _gaMushroomList[i]
			if _gaMushroomList[i].GetHealth() <= 0
				_gaMushroomList[i] = Null
				
			EndIf
			
		EndIf
		
	Next
	
	_fUpdateMushroomListLength()
	
End

'#EndRegion








'#Region __Projectile
Function _fCreateProjectileFrames:Void()
	_gAnimProjectile = _fCreateAnimation(_gImageSpritesheet,
		[0, 69, 21, 21,
		23, 69, 21, 21,
		46, 69, 21, 21,
		69, 69, 21, 21,
		92, 69, 21, 21],
		0)
		
	_gAnimProjectile.SetAuto(0)
	
End




Function _fCreateProjectile:Void(x:Float, y:Float, speed:Float, direction:Int)
	_gaProjectileList = _gaProjectileList.Resize(_gaProjectileList.Length() + 1)
	_gaProjectileList[_gaProjectileList.Length() - 1] = New C_Projectile(x, y, speed, direction)
	_gaProjectileList[_gaProjectileList.Length() - 1].animation.currentFrame = 0
	_gaProjectileList[_gaProjectileList.Length() - 1].animation.frameTimer.SetDelay(100)
	
End




Function _fProjectileMushroomCollisions:Void()
	Local projectile:C_Projectile
	Local mushroom:C_Mushroom
	
	
	For projectile = EachIn _gaProjectileList
		if projectile
			mushroom = _fProjectileMushroomCollide(projectile)
			
			if mushroom
				projectile.SetActive(0)
				mushroom.DecreaseHealth()
				
			EndIf
			
		EndIf
		
	Next
	
End




Function _fProjectileMushroomCollide:C_Mushroom(projectile:C_Projectile)
	Local i:Int
	Local mushroom:C_Mushroom
	Local trim:Float = 10
	Local xTrim:Float = 0
	Local yTrim:Float = 0
	
	
	if projectile.GetDirection() = 0 or projectile.GetDirection() = 2
		xTrim = trim
				
	EndIf
			
	if projectile.GetDirection() = 1 or projectile.GetDirection() = 3
		yTrim = trim
				
	EndIf
	
	
	For i = 0 to (_gaMushroomList.Length() - 1)
		if projectile.GetAbsX() > (_gaMushroomList[i].GetAbsX() - projectile.GetWidth()) + xTrim And projectile.GetAbsX() < (_gaMushroomList[i].GetAbsX() + _gaMushroomList[i].GetWidth()) - xTrim
			if projectile.GetAbsY() > (_gaMushroomList[i].GetAbsY() - projectile.GetHeight()) + yTrim And projectile.GetAbsY() < (_gaMushroomList[i].GetAbsY() + _gaMushroomList[i].GetHeight()) - yTrim
				mushroom = _gaMushroomList[i]
				
			EndIf
			
		EndIf
		
	Next
	
	Return mushroom
	
End




Function _fProjectileCentipedeCollisions:Void()
	Local projectile:C_Projectile
	Local cPart:C_CentepedePart
	
	
	For projectile = EachIn _gaProjectileList
		if projectile
			cPart = _fProjectileCentipedePartCollide(projectile)
			
			if cPart
				projectile.SetActive(0)
				cPart.SetActive(0)
				
			EndIf
			
		EndIf
		
	Next
	
End




Function _fProjectileCentipedePartCollide:C_CentepedePart(projectile:C_Projectile)
	Local centipede:C_Centipede
	Local cPart:C_CentepedePart
	Local cPartPick:C_CentepedePart
	Local trim:Float = 10
	Local xTrim:Float = 0
	Local yTrim:Float = 0
	
	
	if projectile.GetDirection() = 0 or projectile.GetDirection() = 2
		xTrim = trim
				
	EndIf
			
	if projectile.GetDirection() = 1 or projectile.GetDirection() = 3
		yTrim = trim
				
	EndIf
	
	
	For centipede = EachIn _gaCentipedeList
		For cPart = EachIn centipede.partsList
			if projectile.GetAbsX() > (cPart.GetAbsX() - projectile.GetWidth()) + xTrim And projectile.GetAbsX() < (cPart.GetAbsX() + cPart.GetWidth()) - xTrim
				if projectile.GetAbsY() > (cPart.GetAbsY() - projectile.GetHeight()) + yTrim And projectile.GetAbsY() < (cPart.GetAbsY() + cPart.GetHeight()) - yTrim
					cPartPick = cPart
					Exit
					
				EndIf
				
			EndIf
			
		Next
		
		if cPartPick
			Exit
			
		EndIf
		
	Next
	
	Return cPartPick
	
End




Function _fDrawProjectiles:Void()
	Local projectile:C_Projectile
	
	
	For projectile = EachIn _gaProjectileList
		if projectile
			projectile.Draw()
			
		EndIf
		
	Next
	
End




Function _fUpdateProjectileListLength:Void()
	Local projectile:C_Projectile
	Local pList:C_Projectile[]
	
	
	For projectile = EachIn _gaProjectileList
		if projectile
			pList = pList.Resize(pList.Length() + 1)
			pList[pList.Length() - 1] = projectile
			
		EndIf
		
	Next
	
	_gaProjectileList = []
	
	For projectile = EachIn pList
		if projectile
			_gaProjectileList = _gaProjectileList.Resize(_gaProjectileList.Length() + 1)
			_gaProjectileList[_gaProjectileList.Length() - 1] = projectile
			
		EndIf
		
	Next
	
	pList = []
	
End




Function _fRestrictProjectiles:Void()
	Local i:Int
	
	
	For i = 0 to (_gaProjectileList.Length() - 1)
		if _gaProjectileList[i]
			_gaProjectileList[i].Update(_gDelta)
			
			if _gaProjectileList[i].GetAbsX() < (0 - _gaProjectileList[i].GetWidth()) or _gaProjectileList[i].GetAbsX() > _cGameWidth
				_gaProjectileList[i] = Null
				Exit
				
			EndIf
			
			if _gaProjectileList[i].GetAbsY() < (0 - _gaProjectileList[i].GetHeight()) or _gaProjectileList[i].GetAbsY() > _cGameHeight
				_gaProjectileList[i] = Null
				Exit
				
			EndIf
			
		EndIf
		
		if _gaProjectileList[i]
			if _gaProjectileList[i].GetActive() = 0
				_gaProjectileList[i] = Null
				
			EndIf
			
		EndIf
		
	Next
	
End




Function _fUpdateProjectiles:Void()
	Local projectile:C_Projectile
	
	
	_fRestrictProjectiles()
	_fUpdateProjectileListLength()
	
	For projectile = EachIn _gaProjectileList
		if projectile
			if projectile.animation.frameTimer.GetTrigger() = True
				projectile.animation.IncreaseCurrentFrame()
				
			EndIf
			
		EndIf
		
	Next
	
End

'#EndRegion








'#Region __Centipede
Function _fCreateCentipedeFrames:Void()
	_gAnimCentipedeHead = _fCreateAnimation(_gImageSpritesheet,
		[0, 0, 21, 21,
		23, 0, 21, 21,
		46, 0, 21, 21,
		23, 0, 21, 21],
		200)
	
	_gAnimCentipedeBody = _fCreateAnimation(_gImageSpritesheet,
		[0, 23, 21, 21,
		23, 23, 21, 21,
		46, 23, 21, 21,
		0, 46, 21, 21,
		23, 46, 21, 21,
		0, 46, 21, 21,
		46, 23, 21, 21,
		23, 23, 21, 21],
		25)
	
	_gAnimCentipedeTail = _fCreateAnimation(_gImageSpritesheet,
		[46, 46, 21, 21],
		0)
	
	_gAnimCentipedeHead.SetAuto(1)
	_gAnimCentipedeBody.SetAuto(1)
	_gAnimCentipedeTail.SetAuto(0)
		
End




Function _fCreateCentipede:Void(x:Float, y:Float, direction:Int, length:Int, speed:Float = 0.00, progressionX:Int, progressionY:Int)
	_gaCentipedeList = _gaCentipedeList.Resize(_gaCentipedeList.Length() + 1)
	_gaCentipedeList[_gaCentipedeList.Length() - 1] = New C_Centipede(x, y, direction, length, speed, progressionX, progressionY)
	
End




Function _fCentipedeMushroomCollisions:Void()
	Local centipede:C_Centipede
	Local cPart:C_CentepedePart
	Local mushroom:C_Mushroom
	Local directionChanged:Int = 0
	
	
	For centipede = EachIn _gaCentipedeList
		if centipede
			For cPart = EachIn centipede.partsList
				if cPart
					mushroom = _fCentipedePartMushroomCollide(cPart)
					
					if mushroom
						if cPart.GetDirection() = 0 And directionChanged = 0
							cPart.SetBaseY(mushroom.GetAbsY() + mushroom.GetHeight())
							cPart.SetDirection(cPart.GetProgressionX())
							
							cPart.previousY = cPart.GetAbsY()
							directionChanged = 1
							
						EndIf
						
						if cPart.GetDirection() = 1 And directionChanged = 0
							cPart.SetBaseX(mushroom.GetAbsX() + mushroom.GetWidth())
							cPart.SetDirection(cPart.GetProgressionY())
							
							cPart.previousY = cPart.GetAbsY()
							directionChanged = 1
							
						EndIf
						
						if cPart.GetDirection() = 2 And directionChanged = 0
							cPart.SetBaseY(mushroom.GetAbsY() - cPart.GetHeight())
							cPart.SetDirection(cPart.GetProgressionX())
							
							cPart.previousY = cPart.GetAbsY()
							directionChanged = 1
							
						EndIf
						
						if cPart.GetDirection() = 3 And directionChanged = 0
							cPart.SetBaseX(mushroom.GetAbsX() - cPart.GetWidth())
							cPart.SetDirection(cPart.GetProgressionY())
							
							cPart.previousY = cPart.GetAbsY()
							directionChanged = 1
							
						EndIf
						
					EndIf
					
				EndIf
				
			Next
			
		EndIf
		
	Next
	
End




Function _fCentipedePartMushroomCollide:C_Mushroom(cPart:C_CentepedePart)
	Local mushroom:C_Mushroom
	Local flag:C_Mushroom
	
	
	For mushroom = EachIn _gaMushroomList
		if cPart.GetAbsX() > (mushroom.GetAbsX() - cPart.GetWidth()) And cPart.GetAbsX() < (mushroom.GetAbsX() + mushroom.GetWidth())
			if cPart.GetAbsY() > (mushroom.GetAbsY() - cPart.GetHeight()) And cPart.GetAbsY() < (mushroom.GetAbsY() + mushroom.GetHeight())
				flag = mushroom
				
			EndIf
			
		EndIf
		
	Next
	
	Return flag
	
End




Function _fCentipedeRestrictToFrame:Void()
	Local top:Float
	Local bottom:Float
	Local left:Float
	Local right:Float
	Local centipede:C_Centipede
	Local cPart:C_CentepedePart
	
	
	For centipede = EachIn _gaCentipedeList
		if centipede
			For cPart = EachIn centipede.partsList
				if cPart
					top = 0
					bottom = (cPart.GetHeight() * 27)
					left = 0
					right = (cPart.GetWidth() * 37)
					
					if cPart.GetAbsY() = top
						cPart.SetProgressionY(2)
						
					EndIf
					
					if cPart.GetAbsY() = bottom
						cPart.SetProgressionY(0)
						
					EndIf
					
					if cPart.GetDirection() = 0
						if cPart.GetAbsY() <= (cPart.previousY - cPart.GetHeight())
							cPart.SetBaseY(cPart.previousY - cPart.GetHeight())
							
							if cPart.GetAbsX() = left
								cPart.SetDirection(3)
								
							EndIf
							
							if cPart.GetAbsX() = right
								cPart.SetDirection(1)
								
							EndIf
							
						EndIf
						
						if cPart.GetAbsY() < top
							cPart.SetBaseY(top)
							
							if cPart.GetAbsX() = left
								cPart.SetDirection(3)
								
							EndIf
						
							if cPart.GetAbsX() = right
								cPart.SetDirection(1)
								
							EndIf
							
						EndIf
						
					EndIf
					
					if cPart.GetDirection() = 1
						if cPart.GetAbsX() <= left
							cPart.SetBaseX(left)
							cPart.SetDirection(2)
							cPart.previousY = cPart.GetAbsY()
							cPart.SetDirection(cPart.GetProgressionY())
							
						EndIf
						
					EndIf
					
					if cPart.GetDirection() = 2
						if cPart.GetAbsY() >= (cPart.previousY + cPart.GetHeight())
							cPart.SetBaseY(cPart.previousY + cPart.GetHeight())
							
							if cPart.GetAbsX() = left
								cPart.SetDirection(3)
								
							EndIf
							
							if cPart.GetAbsX() = right
								cPart.SetDirection(1)
								
							EndIf
							
						EndIf
						
						if cPart.GetAbsY() >= bottom
							cPart.SetBaseY(bottom)
							
							if cPart.GetAbsX() = left
								cPart.SetDirection(3)
								
							EndIf
							
							if cPart.GetAbsX() = right
								cPart.SetDirection(1)
								
							EndIf
							
						EndIf
						
					EndIf
					
					if cPart.GetDirection() = 3
						if cPart.GetAbsX() >= right
							cPart.SetBaseX(right)
							cPart.SetDirection(2)
							cPart.previousY = cPart.GetAbsY()
							cPart.SetDirection(cPart.GetProgressionY())
							
						EndIf
						
					EndIf
					
				EndIf
			Next
			
		EndIf
		
	Next
		
End




Function _fDrawCentipedes:Void()
	Local centipede:C_Centipede
	
	
	For centipede = EachIn _gaCentipedeList
		centipede.Draw()
		
	Next
	
End




Function _fUpdateCentipedeList:Void()
	Local centipede:C_Centipede
	Local cList:C_Centipede[]
	
	
	For centipede = EachIn _gaCentipedeList
		if centipede
			if centipede.GetActive() = 1
				cList = cList.Resize(cList.Length() + 1)
				cList[cList.Length() - 1] = centipede
				
			EndIf
			
		EndIf
		
	Next
	
	_gaCentipedeList = []
	
	For centipede = EachIn cList
		_gaCentipedeList = _gaCentipedeList.Resize(_gaCentipedeList.Length() + 1)
		_gaCentipedeList[_gaCentipedeList.Length() - 1] = centipede
		
	Next
	
	cList = []
	
End




Function _fUpdateCentipedePartLists:Void()
	Local centipede:C_Centipede
	Local cPartIndex:Int
	Local cpListTemp:C_CentepedePart[]
	
	
	For centipede = EachIn _gaCentipedeList
		if centipede
			_fCreateDebugMessage("Centipede Exists!")
			
			For cPartIndex = 0 to (centipede.partsList.Length() - 1)
				if centipede.partsList[cPartIndex]
					if centipede.partsList[cPartIndex].GetActive() = 1
						cpListTemp = cpListTemp.Resize(cpListTemp.Length() + 1)
						cpListTemp[cpListTemp.Length() - 1] = centipede.partsList[cPartIndex]
						
					EndIf
					
					if centipede.partsList[cPartIndex].GetActive() = 0
						if centipede.partsList[cPartIndex - 1]
							centipede.partsList[cPartIndex - 1].SetAnim(_gAnimCentipedeTail)
							
						EndIf
						
						if centipede.partsList[cPartIndex + 1]
							centipede.partsList[cPartIndex + 1].SetAnim(_gAnimCentipedeHead)
							
						EndIf
						
						_fCreateMushroom(centipede.partsList[cPartIndex].GetAbsX(), centipede.partsList[cPartIndex].GetAbsY())
						
					EndIf
					
				EndIf
				
			Next
			
			centipede.partsList = []
			
			For cPartIndex = 0 to (cpListTemp.Length() - 1)
				centipede.partsList = centipede.partsList.Resize(centipede.partsList.Length() + 1)
				centipede.partsList[centipede.partsList.Length() - 1] = cpListTemp[cPartIndex]
				
			Next
			
		EndIf
		
	Next
	
	cpListTemp = []
	
End




Function _fUpdateCentipedes:Void(dlt:Float)
	Local centipede:C_Centipede
	Local cPart:C_CentepedePart
	
	
	_fUpdateCentipedePartLists()
	_fUpdateCentipedeList()
	
	For centipede = EachIn _gaCentipedeList
		For cPart = EachIn centipede.partsList
			if cPart.GetDirection() = 0 or cPart.GetDirection() = 2
				if (cPart.GetAbsY() Mod cPart.GetHeight()) = 0 And cPart.GetAbsY() <> cPart.previousY
					cPart.SetDirection(cPart.GetProgressionX())
					
				EndIf
				
			EndIf
			
		Next
		
		centipede.Update(dlt)
		
	Next
	
End

'#End Region







'#Region __Player
Function _fCreatePlayerFrames:Void()
	_gAnimPlayer = _fCreateAnimation(_gImageSpritesheet,
		[0, 92, 21, 21],
		0)
		
	_gAnimPlayer.SetAuto(0)
	
End




Function _fCreatePlayer:Void(x:Float, y:Float, direction:Int, speed:Float)
	if _gPlayer = Null
		_gPlayer = New C_Player()
		
	EndIf
	
	_gPlayer.SetAbsX(x)
	_gPlayer.SetAbsY(y)
	_gPlayer.SetDirection(direction)
	_gPlayer.SetSpeed(speed)
	
End




Function _fPlayerExistsAtPos:Int(x:Float, y:Float)
	Local flag:Int
	
	
	If _gPlayer
		if x > _gPlayer.GetAbsX() And x < (_gPlayer.GetAbsX() + _gPlayer.GetWidth())
			If y > _gPlayer.GetAbsY() And y < (_gPlayer.GetAbsY() + _gPlayer.GetHeight())
				flag = 1
				
			Else
				flag = 0
				
			EndIf
			
		EndIf
		
	EndIf
	
	Return flag
	
End




Function _fControlPlayer:Void(dlt:Float = _gDelta)
	Local pixelsPerSecond:Float = (dlt * 100)
	Local speedUp:Int = 0
	Local direction:Int = 255
	
	
	If _fKeyLocked(KEY_UP) = True or _fKeyLocked(KEY_W) = True
		direction = 0
		
	EndIf
	
	If _fKeyLocked(KEY_LEFT) = True or _fKeyLocked(KEY_A) = True
		direction = 1
		
	EndIf
	
	If _fKeyLocked(KEY_DOWN) = True or _fKeyLocked(KEY_S) = True
		direction = 2
		
	EndIf
	
	If _fKeyLocked(KEY_RIGHT) = True or _fKeyLocked(KEY_D) = True
		direction = 3
		
	EndIf
	
	
	if direction <> _gPlayer.GetDirection()
		speedUp = 0
		_gPlayer.SetSpeed(1)
		
	EndIf
	
	
	if direction <> 255
		_gPlayer.SetDirection(direction)
		
	EndIf
	
	if direction = 0
		_gPlayer.SetAbsY(_gPlayer.GetAbsY() - (pixelsPerSecond * _gPlayer.GetSpeed()))
		_gPlayer.SetSpeed(_gPlayer.GetSpeed() + 0.1)
		speedUp = 1
		
	EndIf
	
	if direction = 1
		_gPlayer.SetAbsX(_gPlayer.GetAbsX() - (pixelsPerSecond * _gPlayer.GetSpeed()))
		_gPlayer.SetSpeed(_gPlayer.GetSpeed() + 0.1)
		speedUp = 1
		
	EndIf
	
	if direction = 2
		_gPlayer.SetAbsY(_gPlayer.GetAbsY() + (pixelsPerSecond * _gPlayer.GetSpeed()))
		_gPlayer.SetSpeed(_gPlayer.GetSpeed() + 0.1)
		speedUp = 1
		
	EndIf
	
	if direction = 3
		_gPlayer.SetAbsX(_gPlayer.GetAbsX() + (pixelsPerSecond * _gPlayer.GetSpeed()))
		_gPlayer.SetSpeed(_gPlayer.GetSpeed() + 0.1)
		speedUp = 1
		
	EndIf
	
	if _gPlayer.GetSpeed() > 5.00
		_gPlayer.SetSpeed(5)
		
	EndIf
	
	if _fKeyTapped(KEY_CONTROL) = 1
		_fCreateProjectile(_gPlayer.GetAbsX(), _gPlayer.GetAbsY(), 10, _gPlayer.GetDirection())
		
	EndIf
	
End




Function _fPlayerMushroomCollisions:Void()
	Local mushroom:C_Mushroom
	
	
	mushroom = _fPlayerMushroomCollide(_gPlayer)
	
	if mushroom
		if _gPlayer.GetDirection() = 0
			_gPlayer.SetAbsY(mushroom.GetAbsY() + mushroom.GetHeight())
			
		EndIf
		
		if _gPlayer.GetDirection() = 1
			_gPlayer.SetAbsX(mushroom.GetAbsX() + mushroom.GetWidth())
			
		EndIf
		
		if _gPlayer.GetDirection() = 2
			_gPlayer.SetAbsY(mushroom.GetAbsY() - _gPlayer.GetHeight())
			
		EndIf
		
		if _gPlayer.GetDirection() = 3
			_gPlayer.SetAbsX(mushroom.GetAbsX() - _gPlayer.GetWidth())
			
		EndIf
		
	EndIf
	
End




Function _fPlayerMushroomCollide:C_Mushroom(player:C_Player)
	Local i:Int
	Local mushroom:C_Mushroom
	
	
	For i = 0 to (_gaMushroomList.Length() - 1)
		if player.GetAbsX() > (_gaMushroomList[i].GetAbsX() - player.GetWidth()) And player.GetAbsX() < (_gaMushroomList[i].GetAbsX() + _gaMushroomList[i].GetWidth())
			if player.GetAbsY() > (_gaMushroomList[i].GetAbsY() - player.GetHeight()) And player.GetAbsY() < (_gaMushroomList[i].GetAbsY() + _gaMushroomList[i].GetHeight())
				mushroom = _gaMushroomList[i]
				
			EndIf
			
		EndIf
		
	Next
	
	Return mushroom
	
End




Function _fRestrictPlayer:Void()
	Local mushroom:C_Mushroom
	
	
	if _gPlayer.GetAbsX() <= 0
		_gPlayer.SetAbsX(0)
		
	EndIf
	
	if _gPlayer.GetAbsY() <= 0
		_gPlayer.SetAbsY(0)
		
	EndIf
	
	if _gPlayer.GetAbsX() >= (_cGameWidth - _gPlayer.GetWidth())
		_gPlayer.SetAbsX(_cGameWidth - _gPlayer.GetWidth())
		
	EndIf
	
	if _gPlayer.GetAbsY() >= (_cGameHeight - _gPlayer.GetHeight())
		_gPlayer.SetAbsY(_cGameHeight - _gPlayer.GetHeight())
		
	EndIf
	
End




Function _fDrawPlayer:Void()
	_gPlayer.Draw()
	
End




Function _fUpdatePlayer:Void()
	_fRestrictPlayer()
	_fPlayerMushroomCollisions()
	
End

'#EndRegion








'#Region __Graphics
Function _fLoadGraphics:Void()
	_gImageSpritesheet = LoadImage("graphics/spritesheet.png")
	
End




Function _fUnloadGraphics:Void()
	_gImageSpritesheet = Null
	
End




Function _fCreateAnimation:C_Animation(sourceImage:Image, frameList:Int[], frameTime:Float)
	Return New C_Animation(sourceImage, frameList, frameTime)
	
End




Function _fFrameGrab:Image(sourceImage:Image, x:Int, y:Int, width:Float, height:Float, midhandle:Int)
	Return sourceImage.GrabImage(x, y, width, height, 1, midhandle)
	
End




Function _fCreateAnimations:Void()
	_fCreateCentipedeFrames()
	_fCreateMushroomFrames()
	_fCreateProjectileFrames()
	_fCreatePlayerFrames()
	
End




Function _fDrawImageToScale:Void(sourceImage:Image, x:Float, y:Float, rotation:Float)
	x = (x * _fGetScaleX())
	y = (y * _fGetScaleY())
	
	DrawImage(sourceImage, x, y, rotation, _fGetScaleX(), _fGetScaleY())
	
End




Function _fDrawTextAtScale:Void(txt:String, x:Float, y:Float)
	x = (x * _fGetScaleX())
	y = (y * _fGetScaleY())
	
	DrawText(txt, x, y)
	
End




Function _fGetScaleX:Float()
	Return DeviceWidth() / _cGameWidth
	
End




Function _fGetScaleY:Float()
	Return DeviceHeight() / _cGameHeight
	
End




Function _fDrawSquare:Void(x:Float, y:Float, width:Float, height:Float)
	x = (x * _fGetScaleX())
	y = (y * _fGetScaleY())
	width = (width * _fGetScaleX())
	height = (height * _fGetScaleY())
	
	DrawLine(x, y, x + width, y)
	DrawLine(x, y + width, x + width, y + height)
	DrawLine(x, y, x, y + height)
	DrawLine(x + width, y, x + width, y + height)
	
End




Function _fRectsCollide:Int(rect1:C_Rectangle, rect2:C_Rectangle)
	Local flag:Int = 0
	
	
	if rect1.GetAbsX() > (rect2.GetAbsX() - rect1.GetWidth()) And rect1.GetAbsX() < (rect2.GetAbsX() + rect2.GetWidth())
		if rect1.GetAbsY() > (rect2.GetAbsY() - rect1.GetHeight()) and rect1.GetAbsY() < (rect2.GetAbsY() + rect2.GetHeight())
			flag = 1
			
		EndIf
		
	EndIf
	
	Return flag
	
End

'#End Region







'#Region __Game
Function _fInitiate:Void()
	_fSetGameOn(1)
	
End




Function _fSetGameOn:Void(state:Int)
	_gGameOn = state
	
End




Function _fGetGameOn:Int()
	Return _gGameOn
	
End




Function _fDrawGame:Void()
	_fDrawMushrooms()
	_fDrawCentipedes()
	_fDrawProjectiles()
	_fDrawPlayer()
	
End




Function _fUpdateGame:Void(dlt:Float = _gDelta)
	_fControlPlayer()
	_fCentipedeRestrictToFrame()
	_fCentipedeMushroomCollisions()
	_fProjectileMushroomCollisions()
	_fProjectileCentipedeCollisions()
	_fUpdateMushrooms()
	_fUpdateProjectiles()
	_fUpdateCentipedes(dlt)
	_fUpdatePlayer()
	
End

'#EndRegion







'#Region __Title

'#EndRegion







'#Region __Input
Function _fKeyTapped:Bool(keyCode:Int)
	Local flag:Bool = False
	
	
	if _gaKeyTappedRegister[keyCode] = 0
		if KeyDown(keyCode) = True
			_gaKeyTappedRegister[keyCode] = 1
			flag = True
			
		EndIf
		
	EndIf
	
	if KeyDown(keyCode) = False
		_gaKeyTappedRegister[keyCode] = 0
		
	EndIf
	
	Return flag
	
End




Function _fKeyLocked:Bool(keyCode:Int)
	Local flag:Bool = False
	
	if KeyDown(_gKeyLockedRegister) = 0
		_gKeyLockedRegister = 0
		
	EndIf
	
	if KeyDown(keyCode) = 1 And _gKeyLockedRegister = 0
		_gKeyLockedRegister = keyCode
		
	EndIf
	
	if _gKeyLockedRegister = keyCode
		flag = True
		
	EndIf
	
	Return flag
	
End

'#EndRegion




'#Region __Debugging
Function _fCreateDebugMessage:Void(var:String)
	gDebugMessage = gDebugMessage.Resize(gDebugMessage.Length() + 1)
	gDebugMessage[gDebugMessage.Length() - 1] = var
	
End




Function _fDrawDebugMessages:Void(x:Float, y:Float)
	Local i:Int
	
	
	DrawText("Debug Mode!", 0, 0)
	
	For i = 0 to (gDebugMessage.Length() - 1)
		DrawText(gDebugMessage[i], x, ((i + 1) * 15) + y)
		gDebugMessage[i] = ""
		
	Next
	
	gDebugMessage = gDebugMessage.Resize(0)
	
End

'#EndRegion




